@@ -17,7 +17,7 @@
         decent-to-good ratio, good portability, and don't mind waiting.
 
     "lzf"
-        Custom compression filter for h5py.  This filter is much, much faster 
+        Custom compression filter for h5py.  This filter is much, much faster
         than gzip (roughly 10x in compression vs. gzip level 4, and 3x faster
         in decompressing), but at the cost of a worse compression ratio.  Use
         this if you want cheap compression and portability is not a concern.
@@ -35,13 +35,15 @@
         Tuple of available filter names for decoding
 
     encode
-        Tuple of available filter names for encoding 
+        Tuple of available filter names for encoding
 """
 
 from __future__ import absolute_import, division
 
 import numpy as np
-from .. import h5z, h5p, h5d
+import six
+from .compat import filename_encode
+from .. import h5z, h5p, h5d, h5f
 
 
 _COMP_FILTERS = {'gzip': h5z.FILTER_DEFLATE,
@@ -70,8 +72,39 @@ def _gen_filter_tuples():
 
 decode, encode = _gen_filter_tuples()
 
-def generate_dcpl(shape, dtype, chunks, compression, compression_opts,
-                  shuffle, fletcher32, maxshape, scaleoffset):
+def _external_entry(name, offset=0, size=h5f.UNLIMITED):
+    """ Check for and return a well-formed entry tuple for
+    a call to h5p.set_external. """
+    if not isinstance(name, six.string_types):
+        raise TypeError("External entry's name must be a string")
+    if not isinstance(offset, six.integer_types):
+        raise TypeError("External entry's offset must be an integer")
+    if not isinstance(size, six.integer_types):
+        raise TypeError("External entry's size must be an integer")
+    return (filename_encode(name), offset, size)
+
+def _normalize_external(external):
+    """ Normalize external into a well-formed list of tuples and return. """
+    if external is None:
+        return []
+    elif isinstance(external, six.string_types):
+        # accept a solitary file string
+        return [_external_entry(external)]
+    if not isinstance(external, (list)):
+        raise ValueError('external should be a list of tuples of (file[, offset[, size]])')
+    try:
+        # accept a single entry, not in a list
+        return [_external_entry(*external)]
+    except TypeError:
+        pass
+    # check and rebuild each list entry to be well-formed
+    return [_external_entry(entry)
+            if isinstance(entry, six.string_types) else
+            _external_entry(*entry)
+            for entry in external]
+
+def fill_dcpl(plist, shape, dtype, chunks, compression, compression_opts,
+              shuffle, fletcher32, maxshape, scaleoffset, external):
     """ Generate a dataset creation property list.
 
     Undocumented and subject to change without warning.
@@ -133,15 +166,15 @@ def generate_dcpl(shape, dtype, chunks, compression, compression_opts,
     elif compression_opts is not None:
         # Can't specify just compression_opts by itself.
         raise TypeError("Compression method must be specified")
-    
+
     if scaleoffset is not None:
         # scaleoffset must be an integer when it is not None or False,
         # except for integral data, for which scaleoffset == True is
         # permissible (will use SO_INT_MINBITS_DEFAULT)
-        
+
         if scaleoffset < 0:
             raise ValueError('scale factor must be >= 0')
-                
+
         if dtype.kind == 'f':
             if scaleoffset is True:
                 raise ValueError('integer scaleoffset must be provided for '
@@ -152,7 +185,7 @@ def generate_dcpl(shape, dtype, chunks, compression, compression_opts,
         else:
             raise TypeError('scale/offset filter only supported for integer '
                             'and floating-point types')
-        
+
         # Scale/offset following fletcher32 in the filter chain will (almost?)
         # always triggers a read error, as most scale/offset settings are
         # lossy. Since fletcher32 must come first (see comment below) we
@@ -160,17 +193,18 @@ def generate_dcpl(shape, dtype, chunks, compression, compression_opts,
         if fletcher32:
             raise ValueError('fletcher32 cannot be used with potentially lossy'
                              ' scale/offset filter')
+
+    external = _normalize_external(external)
     # End argument validation
 
     if (chunks is True) or \
-    (chunks is None and any((shuffle, fletcher32, compression, maxshape, 
+    (chunks is None and any((shuffle, fletcher32, compression, maxshape,
                              scaleoffset is not None))):
         chunks = guess_chunk(shape, maxshape, dtype.itemsize)
-        
+
     if maxshape is True:
         maxshape = (None,)*len(shape)
 
-    plist = h5p.create(h5p.DATASET_CREATE)
     if chunks is not None:
         plist.set_chunk(chunks)
         plist.set_fill_time(h5d.FILL_TIME_ALLOC)  # prevent resize glitch
@@ -182,6 +216,9 @@ def generate_dcpl(shape, dtype, chunks, compression, compression_opts,
         else: # dtype.kind == 'f'
             plist.set_scaleoffset(h5z.SO_FLOAT_DSCALE, scaleoffset)
 
+    for item in external:
+        plist.set_external(*item)
+
     if shuffle:
         plist.set_shuffle()
 
@@ -260,7 +297,7 @@ def guess_chunk(shape, maxshape, typesize):
     Undocumented and subject to change without warning.
     """
     # pylint: disable=unused-argument
-    
+
     # For unlimited dimensions we have to guess 1024
     shape = tuple((x if x!=0 else 1024) for i, x in enumerate(shape))
 
@@ -303,11 +340,3 @@ def guess_chunk(shape, maxshape, typesize):
         idx += 1
 
     return tuple(int(x) for x in chunks)
-
-
-
-
-
-
-
-
