@@ -2,16 +2,16 @@
 """
     Implements a new custom Distutils command for handling library
     configuration.
-    
+
     The "configure" command here doesn't directly affect things like
     config.pxi; rather, it exists to provide a set of attributes that are
     used by the build_ext replacement in setup_build.py.
-    
+
     Options from the command line and environment variables are stored
     between invocations in a pickle file.  This allows configuring the library
     once and e.g. calling "build" and "test" without recompiling everything
     or explicitly providing the same options every time.
-    
+
     This module also contains the auto-detection logic for figuring out
     the currently installed HDF5 version.
 """
@@ -54,7 +54,7 @@ class EnvironmentOptions(object):
     """
         Convenience class representing the current environment variables.
     """
-    
+
     def __init__(self):
         self.hdf5 = os.environ.get('HDF5_DIR')
         self.hdf5_version = os.environ.get('HDF5_VERSION')
@@ -68,15 +68,15 @@ class configure(Command):
     """
         Configure build options for h5py: custom path to HDF5, version of
         the HDF5 library, and whether MPI is enabled.
-    
+
         Options come from the following sources, in order of priority:
-        
+
         1. Current command-line options
         2. Old command-line options
         3. Current environment variables
         4. Old environment variables
         5. Autodetection
-        
+
         When options change, the rebuild_required attribute is set, and
         may only be reset by calling reset_rebuild().  The custom build_ext
         command does this.s
@@ -88,13 +88,13 @@ class configure(Command):
                     ('hdf5-version=', '5', 'HDF5 version "X.Y.Z"'),
                     ('mpi', 'm', 'Enable MPI building'),
                     ('reset', 'r', 'Reset config options') ]
-    
+
     def initialize_options(self):
         self.hdf5 = None
         self.hdf5_version = None
         self.mpi = None
         self.reset = None
-        
+
     def finalize_options(self):
         if self.hdf5_version is not None:
             validate_version(self.hdf5_version)
@@ -104,17 +104,17 @@ class configure(Command):
         dct = loadpickle()
         dct['rebuild'] = False
         savepickle(dct)
-        
+
     def run(self):
         """ Distutils calls this when the command is run """
-        
+
         env = EnvironmentOptions()
-                
+
         # Step 1: determine if settings have changed and update cache
-        
+
         oldsettings = {} if self.reset else loadpickle()
         dct = oldsettings.copy()
-        
+
         # Only update settings which have actually been specified this
         # round; ignore the others (which have value None).
         if self.hdf5 is not None:
@@ -131,25 +131,25 @@ class configure(Command):
             dct['env_mpi'] = env.mpi
 
         self.rebuild_required = dct.get('rebuild') or dct != oldsettings
-        
+
         # Corner case: rebuild if options reset, but only if they previously
         # had non-default values (to handle multiple resets in a row)
         if self.reset and any(loadpickle().values()):
             self.rebuild_required = True
-            
+
         dct['rebuild'] = self.rebuild_required
-        
+
         savepickle(dct)
-    
+
         # Step 2: update public config attributes according to priority rules
-          
+
         if self.hdf5 is None:
             self.hdf5 = oldsettings.get('cmd_hdf5')
         if self.hdf5 is None:
             self.hdf5 = env.hdf5
         if self.hdf5 is None:
             self.hdf5 = oldsettings.get('env_hdf5')
-            
+
         if self.hdf5_version is None:
             self.hdf5_version = oldsettings.get('cmd_hdf5_version')
         if self.hdf5_version is None:
@@ -157,20 +157,16 @@ class configure(Command):
         if self.hdf5_version is None:
             self.hdf5_version = oldsettings.get('env_hdf5_version')
         if self.hdf5_version is None:
-            try:
-                self.hdf5_version = autodetect_version(self.hdf5)
-                print("Autodetected HDF5 %s" % self.hdf5_version)
-            except Exception as e:
-                sys.stderr.write("Autodetection skipped [%s]\n" % e)
-                self.hdf5_version = '1.8.4'
-                
+            self.hdf5_version = autodetect_version(self.hdf5)
+            print("Autodetected HDF5 %s" % self.hdf5_version)
+
         if self.mpi is None:
             self.mpi = oldsettings.get('cmd_mpi')
         if self.mpi is None:
             self.mpi = env.mpi
         if self.mpi is None:
             self.mpi = oldsettings.get('env_mpi')
-                
+
         # Step 3: print the resulting configuration to stdout
 
         print('*' * 80)
@@ -193,21 +189,23 @@ def autodetect_version(hdf5_dir=None):
 
     hdf5_dir: optional HDF5 install directory to look in (containing "lib")
     """
-
-    import os
-    import sys
-    import os.path as op
     import re
     import ctypes
     from ctypes import byref
 
     import pkgconfig
-    
+
     if sys.platform.startswith('darwin'):
+        default_path = 'libhdf5.dylib'
         regexp = re.compile(r'^libhdf5.dylib')
+    elif sys.platform.startswith('win') or \
+        sys.platform.startswith('cygwin'):
+        default_path = 'hdf5.dll'
+        regexp = re.compile(r'^hdf5.dll')
     else:
+        default_path = 'libhdf5.so'
         regexp = re.compile(r'^libhdf5.so')
-        
+
     libdirs = ['/usr/local/lib', '/opt/local/lib']
     try:
         if pkgconfig.exists("hdf5"):
@@ -215,7 +213,11 @@ def autodetect_version(hdf5_dir=None):
     except EnvironmentError:
         pass
     if hdf5_dir is not None:
-        libdirs.insert(0, op.join(hdf5_dir, 'lib'))
+        if sys.platform.startswith('win'):
+            lib = 'bin'
+        else:
+            lib = 'lib'
+        libdirs.insert(0, op.join(hdf5_dir, lib))
 
     path = None
     for d in libdirs:
@@ -230,7 +232,9 @@ def autodetect_version(hdf5_dir=None):
             break
 
     if path is None:
-        path = "libhdf5.so"
+        path = default_path
+
+    print("Loading library to get version:", path)
 
     lib = ctypes.cdll.LoadLibrary(path)
 
